package javax.security.auth.kerberos;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.Vector;
import java.io.ObjectOutputStream;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.ArrayList;
import java.io.ObjectStreamField;
import java.security.Permission;
import java.util.List;
import java.io.Serializable;
import java.security.PermissionCollection;

final class KrbServicePermissionCollection extends PermissionCollection implements Serializable
{
    private transient List<Permission> perms;
    private static final long serialVersionUID = -4118834211490102011L;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public KrbServicePermissionCollection() {
        this.perms = new ArrayList();
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof ServicePermission)) {
            return false;
        }
        ServicePermission servicePermission = (ServicePermission)permission;
        int mask = servicePermission.getMask();
        if (mask == 0) {
            Iterator<Permission> iterator = this.perms.iterator();
            while (iterator.hasNext()) {
                if (((ServicePermission)(Permission)iterator.next()).impliesIgnoreMask(servicePermission)) {
                    return true;
                }
                continue;
            }
            return false;
        }
        int n = 0;
        synchronized (this) {
            for (int size = this.perms.size(), i = 0; i < size; ++i) {
                ServicePermission servicePermission2 = (ServicePermission)this.perms.get(i);
                if ((mask & servicePermission2.getMask()) != 0x0 && servicePermission2.impliesIgnoreMask(servicePermission)) {
                    n |= servicePermission2.getMask();
                    if ((n & mask) == mask) {
                        return true;
                    }
                    mask ^= n;
                }
            }
        }
        return false;
    }
    
    @Override
    public void add(Permission obj) {
        if (!(obj instanceof ServicePermission)) {
            throw new IllegalArgumentException("invalid permission: " + obj);
        }
        if (this.isReadOnly()) {
            throw new SecurityException("attempt to add a Permission to a readonly PermissionCollection");
        }
        synchronized (this) {
            this.perms.add(0, obj);
        }
        return;
    }
    
    @Override
    public Enumeration<Permission> elements() {
        synchronized (this) {
            return Collections.enumeration(this.perms);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Vector vector = new Vector(this.perms.size());
        synchronized (this) {
            vector.addAll(this.perms);
        }
        objectOutputStream.putFields().put("permissions", vector);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        Vector<Object> vector = (Vector)objectInputStream.readFields().get("permissions", null);
        (this.perms = new ArrayList(vector.size())).addAll(vector);
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("permissions", Vector.class) };
    }
}
